@@ -64,23 +64,11 @@ struct App {
     room: String,
     rooms: BTreeMap<RoomId, Box<RoomName>>,
     current_room: Option<RoomId>,
-    //using_config: bool,
     room_id_to_canonical_alias: BTreeMap<RoomId, RoomAliasId>,
     auth_token: Option<String>,
-    joined_rooms: Vec<RoomId>, //message_box_height: u16,
+    joined_rooms: Vec<RoomId>,
+    room_to_direct_name: BTreeMap<RoomId, String>,
 }
-/*
-impl Default for App {
-    fn default() -> App {
-        App {
-            input: String::new(),
-            focus: Focus::Watching,
-            messages: Vec::new(),
-
-        }
-    }
-}
-*/
 
 impl App {
     pub async fn new(
@@ -122,8 +110,7 @@ impl App {
             joined_rooms: Vec::new(),
             auth_token: auth_token,
             room: room,
-            //message_box_height: 3,
-            //client: ruma_client::Client::builder().homeserver_url(home_server.into()).build::<ruma_client::http_client::Reqwest>().await.unwrap(),
+            room_to_direct_name: BTreeMap::new(),
         }
     }
 }
@@ -212,6 +199,7 @@ enum SyncMessage {
         Vec<RoomId>,
         BTreeMap<RoomId, Box<RoomName>>,
         BTreeMap<RoomId, RoomAliasId>,
+        BTreeMap<RoomId, String>,
     ),
     LeaveRoom(RoomId),
     JoinRoom(String),
@@ -271,6 +259,7 @@ async fn run_app<B: Backend>(terminal: &mut Terminal<B>, mut app: App) -> io::Re
                     client.data.joined_rooms.clone(),
                     client.data.room_to_name.clone(),
                     client.data.room_id_to_cananoical_alias.clone(),
+                    client.data.room_to_direct_name.clone(),
                 ))
                 .unwrap();
             }
@@ -480,12 +469,14 @@ async fn run_app<B: Backend>(terminal: &mut Terminal<B>, mut app: App) -> io::Re
                     room_id_list,
                     room_to_name,
                     room_id_to_canonical_alias,
+                    room_to_direct_name,
                 ) = msg
                 {
                     //app.rooms = room_id_list;
                     app.joined_rooms = room_id_list.clone();
                     app.rooms = room_to_name;
                     app.room_id_to_canonical_alias = room_id_to_canonical_alias;
+                    app.room_to_direct_name = room_to_direct_name;
                     for room_id in room_id_list {
                         if !app.messages.contains_key(&room_id) {
                             app.messages.insert(room_id.clone(), Vec::new());
@@ -617,11 +608,16 @@ fn ui<B: Backend>(f: &mut Frame<B>, app: &mut App) {
             let content: Vec<Spans>;
             if app.rooms.contains_key(&room_id) {
                 content = vec![Spans::from(Span::raw(format!("{}", app.rooms[room_id])))];
-            } else {
+            } else if app.room_id_to_canonical_alias.contains_key(&room_id) {
                 content = vec![Spans::from(Span::raw(format!(
                     "{}",
                     app.room_id_to_canonical_alias[room_id]
                 )))];
+            } else {
+                content = vec![Spans::from(Span::raw(format!(
+                    "{}",
+                    app.room_to_direct_name[room_id]
+                )))];
             }
             //let content: Vec<Span>;
             if app.current_room.is_none() {
